import requests

from . import API_BASE_URL

from .exceptions import APIError, AuthenticationError, ValidationError


class Client(object):

    def __init__(self,
                 access_token=None,
                 api_base_url=None):

        self.api_base_url = api_base_url or API_BASE_URL

        self.session = requests.Session()

        if access_token:
            self.session.headers.update(
                {'Authorization': 'Bearer %s' % access_token}
            )

    def request(
            self,
            method,
            endpoint,
            params=None,
            data=None,
            json=None,
            files=None,
            suffix='/'):
        url = '%s%s%s' % (self.api_base_url, endpoint, suffix)

        response = getattr(self.session, method)(
            url, json=json, params=params, data=data, files=files
        )

        if response.ok:
            return response
        elif response.status_code == 400:
            raise ValidationError(response)
        elif response.status_code == 401:
            raise AuthenticationError(response)
        else:
            raise APIError(response)

    def create_collection(self, name, **data):
        data['name'] = name

        return self.request('post', 'collections', json=data).json()

    def get_collection(self, collection_id):
        return self.request('get', 'collections/%s' % collection_id).json()

    def update_collection(self, collection_id, **data):
        return self.request('patch', 'collections/%s' % collection_id,
                            json=data).json()

    def delete_collection(self, collection_id):
        return self.request('delete', 'collections/%s' % collection_id)

    def my_collections(self):
        return self.request('get', 'my_collections').json()

    def get_collection_images(self, collection_id, limit=100, offset=0):
        return self.request(
            'get',
            'collections/%s/images' % collection_id,
            params={
                'limit': limit,
                'offset': offset
            }
        ).json()

    def add_image(self, collection_id, file, **data):
        files = {'file': open(file, 'rb')}

        return self.request(
            'post',
            'collections/%s/images' % collection_id,
            data=data,
            files=files
        ).json()

    def get_image(self, image_id):
        return self.request('get', 'images/%s' % image_id).json()

    def update_image(self, image_id, **data):
        return self.request('patch', 'images/%s' % image_id,
                            json=data).json()

    def delete_image(self, image_id):
        return self.request('delete', 'images/%s' % image_id)



try:
    from setuptools.core import setup
except ImportError:
    from distutils.core import setup

__version__ = '0.1.1'

setup(
    name='pynv',
    version='0.1.1',
    author='Anton Burnashev & Luke Chang',
    author_email='luke.j.chang@dartmouth.edu',
    url='https://github.com/ljchang/pynv',
    packages=['pynv'],
    license='MIT',
    install_requires=['requests>=2.10.0'],
    description='A Python library for interfacing with http://neurovault.org upload API',
    keywords=['neuroimaging', 'neurovault'],
    classifiers=[
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
    ]
)



class APIError(Exception):
    def __init__(self, response):

        super(APIError, self).__init__(self._extract_text(response))
        self.response = response

    def _extract_text(self, response):
        return self._text_from_detail(response) or response.text

    def _text_from_detail(self, response):
        try:
            errors = response.json()
            return errors['detail']
        except (AttributeError, KeyError, ValueError):
            return None


class AuthenticationError(APIError):
    def __init__(self, response):
        super(AuthenticationError, self).__init__(response)
        self.errors = response.json()


class ValidationError(APIError):
    def __init__(self, response):
        super(ValidationError, self).__init__(response)
        self.errors = response.json()



"""
    pyneurovault_upload
    ~~~~~~~~~~~~~~~~~~~

    A client for interfacing with http://neurovault.org API


    :copyright: (c) 2016 by Luke Chang.
    :license: MIT, see LICENSE for more details.
"""

API_BASE_URL = 'http://neurovault.org/api/'

from .client import Client

__all__ = ['Client']

__author__ = ['Luke Chang', 'Anton Burnashev']
__license__ = 'MIT'
